@@ -23,11 +23,16 @@ emcc can be influenced by a few environment variables:
   EMMAKEN_COMPILER - The compiler to be used, if you don't want the default clang.
 '''
 
+from tools.toolchain_profiler import ToolchainProfiler
+if __name__ == '__main__':
+  ToolchainProfiler.record_process_start()
+
 import os, sys, shutil, tempfile, subprocess, shlex, time, re, logging
 from subprocess import PIPE
 from tools import shared, jsrun, system_libs
 from tools.shared import execute, suffix, unsuffixed, unsuffixed_basename, WINDOWS, safe_move
 from tools.response_file import read_response_file
+import tools.line_endings
 
 # endings = dot + a suffix, safe to test by  filename.endswith(endings)
 C_ENDINGS = ('.c', '.C', '.i')
@@ -437,6 +442,9 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     valid_abspaths = []
     separate_asm = False
     cfi = False
+    # Specifies the line ending format to use for all generated text files.
+    # Defaults to using the native EOL on each platform (\r\n on Windows, \n on Linux&OSX)
+    output_eol = os.linesep
 
     def is_valid_abspath(path_name):
       # Any path that is underneath the emscripten repository root must be ok.
@@ -727,6 +735,16 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
         newargs[i] = ''
       elif newargs[i].startswith("-fsanitize=cfi"):
         cfi = True
+      elif newargs[i] == "--output_eol":
+        if newargs[i+1].lower() == 'windows':
+          output_eol = '\r\n'
+        elif newargs[i+1].lower() == 'linux':
+          output_eol = '\n'
+        else:
+          logging.error('Invalid value "' + newargs[i+1] + '" to --output_eol!')
+          exit(1)
+        newargs[i] = ''
+        newargs[i+1] = ''
 
     if should_exit:
       sys.exit(0)
@@ -1245,6 +1263,8 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       args = [call] + newargs + input_files
       if file_ending.endswith(CXX_ENDINGS):
         args += shared.EMSDK_CXX_OPTS
+      if not shared.Building.can_inline():
+        args.append('-fno-inline-functions')
       args = system_libs.process_args(args, shared.Settings)
       return args
 
@@ -1887,11 +1907,14 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     # Move final output to the js target
     shutil.move(final, js_target)
 
+    generated_text_files_with_native_eols = [js_target]
+
     # Separate out the asm.js code, if asked. Or, if necessary for another option
     if (separate_asm or shared.Settings.BINARYEN) and not shared.Settings.WASM_BACKEND:
       logging.debug('separating asm')
       with misc_temp_files.get_file(suffix='.js') as temp_target:
         subprocess.check_call([shared.PYTHON, shared.path_from_root('tools', 'separate_asm.py'), js_target, asm_target, temp_target])
+        generated_text_files_with_native_eols += [asm_target]
         shutil.move(temp_target, js_target)
 
       # extra only-my-code logic
@@ -1944,7 +1967,8 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       if 'native-wasm' in shared.Settings.BINARYEN_METHOD or 'interpret-binary' in shared.Settings.BINARYEN_METHOD:
         logging.debug('wasm-as (wasm => binary)')
         subprocess.check_call([os.path.join(binaryen_bin, 'wasm-as'), wasm_text_target, '-o', wasm_binary_target])
-        shutil.copyfile(wasm_text_target + '.mappedGlobals', wasm_binary_target + '.mappedGlobals')
+        if os.path.exists(wasm_text_target + '.mappedGlobals'): # TODO: remove once we no longer use .mappedGlobals files at all, as binaryen is moving to https://github.com/WebAssembly/binaryen/issues/675
+          shutil.copyfile(wasm_text_target + '.mappedGlobals', wasm_binary_target + '.mappedGlobals')
 
     # If we were asked to also generate HTML, do that
     if final_suffix == 'html':
@@ -2072,19 +2096,25 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
           xhr.send(null);
 ''' % (os.path.basename(wasm_binary_target), script_inline)
 
-      html = open(target, 'w')
+      html = open(target, 'wb')
       assert (script_src or script_inline) and not (script_src and script_inline)
       if script_src:
         script_replacement = '<script async type="text/javascript" src="%s"></script>' % script_src
       else:
         script_replacement = '<script>\n%s\n</script>' % script_inline
-      html.write(shell.replace('{{{ SCRIPT }}}', script_replacement))
+      html_contents = shell.replace('{{{ SCRIPT }}}', script_replacement)
+      html_contents = tools.line_endings.convert_line_endings(html_contents, '\n', output_eol)
+      html.write(html_contents)
       html.close()
     else: # final_suffix != html
       if proxy_to_worker:
         shutil.move(js_target, js_target[:-3] + '.worker.js') # compiler output goes in .worker.js file
         worker_target_basename = target_basename + '.worker'
-        open(target, 'w').write(open(shared.path_from_root('src', 'webGLClient.js')).read() + '\n' + open(shared.path_from_root('src', 'proxyClient.js')).read().replace('{{{ filename }}}', shared.Settings.PROXY_TO_WORKER_FILENAME or worker_target_basename).replace('{{{ IDBStore.js }}}', open(shared.path_from_root('src', 'IDBStore.js')).read()))
+        target_contents = open(shared.path_from_root('src', 'webGLClient.js')).read() + '\n' + open(shared.path_from_root('src', 'proxyClient.js')).read().replace('{{{ filename }}}', shared.Settings.PROXY_TO_WORKER_FILENAME or worker_target_basename).replace('{{{ IDBStore.js }}}', open(shared.path_from_root('src', 'IDBStore.js')).read())
+        open(target, 'w').write(target_contents)
+
+    for f in generated_text_files_with_native_eols:
+      tools.line_endings.convert_line_endings_in_file(f, os.linesep, output_eol)
 
     log_time('final emitting')
 
@@ -2101,3 +2131,4 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
 
 if __name__ == '__main__':
   run()
+  sys.exit(0)
