@@ -370,8 +370,14 @@ Module['postMainLoop'] = function() {
 
 // Wait to start running until we receive some info from the client
 
-addRunDependency('gl-prefetch');
-addRunDependency('worker-init');
+#if USE_PTHREADS
+if (!ENVIRONMENT_IS_PTHREAD) {
+#endif
+  addRunDependency('gl-prefetch');
+  addRunDependency('worker-init');
+#if USE_PTHREADS
+}
+#endif
 
 // buffer messages until the program starts to run
 
@@ -391,7 +397,7 @@ function messageResender() {
   }
 }
 
-onmessage = function onmessage(message) {
+function onMessageFromMainEmscriptenThread(message) {
   if (!calledMain && !message.data.preMain) {
     if (!messageBuffer) {
       messageBuffer = [];
@@ -460,6 +466,9 @@ onmessage = function onmessage(message) {
       screen.height = Module.canvas.height_ = message.data.height;
       Module.canvas.boundingClientRect = message.data.boundingClientRect;
       document.URL = message.data.URL;
+#if USE_PTHREADS
+      currentScriptUrl = message.data.currentScriptUrl;
+#endif
       window.fireEvent({ type: 'load' });
       removeRunDependency('worker-init');
       break;
@@ -476,6 +485,14 @@ onmessage = function onmessage(message) {
   }
 };
 
+#if USE_PTHREADS
+if (!ENVIRONMENT_IS_PTHREAD) {
+#endif
+  onmessage = onMessageFromMainEmscriptenThread;
+#if USE_PTHREADS
+}
+#endif
+
 function postCustomMessage(data) {
   postMessage({ target: 'custom', userData: data });
 }
