@@ -90,7 +90,13 @@ typedef long suseconds_t;
 
 
 #if defined(__NEED_pthread_attr_t) && !defined(__DEFINED_pthread_attr_t)
+#ifdef __EMSCRIPTEN__
+// For canvas transfer implementation in Emscripten, use an extra 10th control field
+// to pass a pointer to a string denoting the WebGL canvases to transfer.
+typedef struct { union { int __i[10]; unsigned __s[10]; } __u; } pthread_attr_t;
+#else
 typedef struct { union { int __i[9]; unsigned __s[9]; } __u; } pthread_attr_t;
+#endif
 #define __DEFINED_pthread_attr_t
 #endif
 
